@@ -1,5 +1,5 @@
-;;;; $Id: clisp.lisp 660 2011-05-11 13:08:19Z ctian $
-;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.4/backend/clisp.lisp $
+;;;; $Id: clisp.lisp 687 2012-02-27 14:49:55Z ctian $
+;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.5/backend/clisp.lisp $
 
 ;;;; See LICENSE for licensing information.
 
@@ -116,10 +116,11 @@
 (defun socket-connect (host port &key (protocol :stream) (element-type 'character)
                        timeout deadline (nodelay t nodelay-specified)
                        local-host local-port)
-  (declare (ignore nodelay)
-	   (ignorable timeout local-host local-port))
+  (declare (ignorable timeout local-host local-port))
   (when deadline (unsupported 'deadline 'socket-connect))
-  (when nodelay-specified (unsupported 'nodelay 'socket-connect))
+  (when (and nodelay-specified 
+             (not (eq nodelay :if-supported)))
+    (unsupported 'nodelay 'socket-connect))
   (case protocol
     (:stream
      (let ((socket)
