@@ -1,5 +1,5 @@
-;;;; $Id: sbcl.lisp 675 2011-10-01 14:49:26Z ctian $
-;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.4/backend/sbcl.lisp $
+;;;; $Id: sbcl.lisp 687 2012-02-27 14:49:55Z ctian $
+;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.5/backend/sbcl.lisp $
 
 ;;;; See LICENSE for licensing information.
 
@@ -261,8 +261,11 @@ happen. Use with care."
              ;; package today. There's no guarantee the functions
              ;; we need are available, but we can make sure not to
              ;; call them if they aren't
+             (not (eq nodelay :if-supported))
              (not sockopt-tcp-nodelay-p))
     (unsupported 'nodelay 'socket-connect))
+  (when (eq nodelay :if-supported)
+    (setf nodelay t))
 
   (let ((socket (make-instance 'sb-bsd-sockets:inet-socket
                                :type protocol
