@@ -1,5 +1,5 @@
-;;;; $Id: usocket.lisp 671 2011-08-16 07:58:22Z hhubner $
-;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.4/usocket.lisp $
+;;;; $Id: usocket.lisp 687 2012-02-27 14:49:55Z ctian $
+;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.5/usocket.lisp $
 
 ;;;; See LICENSE for licensing information.
 
@@ -529,7 +529,7 @@ Optionally, a different fractional part can be specified."
 
 ;; Documentation for the function
 ;;
-;; (defun SOCKET-CONNECT (host port &key element-type) ..)
+;; (defun SOCKET-CONNECT (host port &key element-type nodelay some-other-keys...) ..)
 ;;
 (setf (documentation 'socket-connect 'function)
       "Connect to `host' on `port'.  `host' is assumed to be a string or
@@ -539,6 +539,20 @@ an IP address represented in vector notation, such as #(192 168 1 1).
 `element-type' specifies the element type to use when constructing the
 stream associated with the socket.  The default is 'character.
 
+`nodelay' Allows to disable/enable Nagle's algorithm (http://en.wikipedia.org/wiki/Nagle%27s_algorithm).
+If this parameter is omitted, the behaviour is inherited from the 
+CL implementation (in most cases, Nagle's algorithm is 
+enabled by default, but for example in ACL it is disabled).
+If the parmeter is specified, one of these three values is possible: 
+  T - Disable Nagle's algorithm; signals an UNSUPPORTED
+      condition if the implementation does not support explicit 
+      manipulation with that option.
+  NIL - Leave Nagle's algorithm enabled on the socket;
+      signals an UNSUPPORTED condition if the implementation does 
+      not support explicit manipulation with that option.
+  :IF-SUPPORTED - Disables Nagle's algorithm if the implementation
+      allows this, otherwises just ignore this option.
+
 Returns a usocket object.")
 
 ;; Documentation for the function
